package GCPlugins::GCboardgames::GCboardgamegeek;

###################################################
#
#  Copyright 2005-2009 Tian
#
#  This file is part of GCstar.
#
#  GCstar is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  GCstar is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with GCstar; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###################################################

use strict;
use utf8;

use GCPlugins::GCboardgames::GCboardgamesCommon;

{
    package GCPlugins::GCboardgames::GCPluginboardgamegeek;

    use base qw(GCPlugins::GCboardgames::GCboardgamesPluginsBase);
 
    sub start
    {
        my ($self, $tagname, $attr, $attrseq, $origtext) = @_;
	
        $self->{inside}->{$tagname}++;

        if ($self->{parsingEnded})
        {
            return;
        }
        
        if ($self->{parsingList})
        {
            if (($tagname eq "h1") && ($attr->{class} eq "geekitem"))
            {
                $self->{parsingEnded} = 1;
                $self->{itemIdx} = 0;
                $self->{itemsList}[0]->{url} = $self->{loadedUrl};
            }
            if ($tagname eq "table")
            {
                if ($attr->{class} eq "collection_table")
                {        
                    $self->{isInsideResultsTable} = 1;
	        }                
            }
            
            if (($tagname eq "td") && ($self->{isInsideResultsTable}))
            {
                if ($attr->{class} =~ /collection_objectname/)
                {        
                    $self->{isInsideResult} = 1;
	        }                
            }

            if (($tagname eq "a") && ($self->{isInsideResult}))
            {
                if ($attr->{href} =~ /^\/boardgame\//)
                {
                    my $url = $attr->{href};
                    $self->{isBoardgame} = 1;
                    $self->{itemIdx}++;
                    $self->{itemsList}[$self->{itemIdx}]->{url} = $url;
                    $self->{temporaryName} = "";
                    $self->{ignoreAltTitle} = 0;
                }
            }

            if (($tagname eq "span") && ($self->{isInsideResult}))
            {
                if ($attr->{class} =~ /smallerfont dull/)
                {
                    $self->{isInsideRelease} = 1;
                }
            }
        }
        else
        {
            if (($tagname eq "h1") && ($attr->{class} eq "geekitem"))
            {
                $self->{insideName} = 1;
            }
            elsif (($tagname eq "span") && ($attr->{id} eq "edityear")) 
            {
		$self->{insideYear} = 1;
            } 
            elsif (($tagname eq "div") && ($attr->{id} =~ m/results_players_thing/)) 
            {
		$self->{insidePlayers} = 1;
            }
            elsif (($tagname eq "div") && ($attr->{id} =~ m/results_boardgamedesigner_thing/)) 
            {
		$self->{insideDesigners} = 1;           
            }
            elsif (($tagname eq "a") && ($attr->{href} =~ /^\/boardgamedesigner\//) && ($self->{insideDesigners})) 
            {
		$self->{insideDesigner} = 1;           
            }
            elsif (($tagname eq "div") && ($attr->{id} =~ m/results_boardgamepublisher_thing/)) 
            {
		$self->{insidePublishers} = 1;           
            }
            elsif (($tagname eq "a") && ($attr->{href} =~ /^\/boardgamepublisher\//) && ($self->{insidePublishers})) 
            {
		$self->{insidePublisher} = 1;           
            }
            elsif (($tagname eq "div") && ($attr->{id} =~ m/results_boardgameartist_thing/)) 
            {
		$self->{insideArtists} = 1;           
            }
            elsif (($tagname eq "a") && ($attr->{href} =~ /^\/boardgameartist\//) && ($self->{insideArtists})) 
            {
		$self->{insideArtist} = 1;           
            }
            elsif (($tagname eq "div") && ($attr->{id} =~ m/results_playingtime_thing/)) 
            {
		$self->{insidePlayingTime} = 1;
            }
            elsif (($tagname eq "div") && ($attr->{id} =~ m/results_minage_thing/)) 
            {
		$self->{insideAges} = 1;
            }
            elsif (($tagname eq "div") && ($attr->{id} =~ m/results_boardgamecategory_thing/)) 
            {                 
		$self->{insideCategories} = 1;
            }
            elsif (($tagname eq "div") && ($attr->{id} =~ m/results_boardgamemechanic_thing/)) 
            {
		$self->{insideMechanics} = 1;
            }
            elsif (($tagname eq "div") && ($attr->{id} =~ m/results_expansions_thing/)) 
            {
		$self->{insideExpansions} = 1;
            }
            elsif (($tagname eq "div") && ($attr->{id} eq "module_4")) 
            {
		$self->{insideInfo} = 1;
            }  
            elsif (($tagname eq "div") && ($attr->{id} eq "module_10")) 
            {
		$self->{insideImages} = 1;
            }      
            elsif (($tagname eq "div") && ($attr->{id} eq "module_5")) 
            {
		$self->{insideDescription} = 1;
            }              
            elsif ($tagname eq "img")
            {
                if ($self->{insideImages})
                {
                    if (! $self->{curInfo}->{photo1} )
                    {
                        $self->{tempPic} = $attr->{src};
                        $self->{tempPic} =~ s/_mt/_md/;
                        $self->{curInfo}->{photo1} = $self->{tempPic}
                    }
                    elsif (! $self->{curInfo}->{photo2} )
                    {
                        $self->{tempPic} = $attr->{src};
                        $self->{tempPic} =~ s/_mt/_md/;
                        $self->{curInfo}->{photo2} = $self->{tempPic}
                    }
                    elsif (! $self->{curInfo}->{photo3} )
                    {
                        $self->{tempPic} = $attr->{src};
                        $self->{tempPic} =~ s/_mt/_md/;
                        $self->{curInfo}->{photo3} = $self->{tempPic}
                    }
                    elsif (! $self->{curInfo}->{photo4} )
                    {
                        $self->{tempPic} = $attr->{src};
                        $self->{tempPic} =~ s/_mt/_md/;
                        $self->{curInfo}->{photo4} = $self->{tempPic}
                    }
                }
                elsif ($self->{insideInfo})
                {
	            $self->{tempPic} = $attr->{src};
                    $self->{tempPic} =~ s/_t/_md/;
                    $self->{curInfo}->{boxpic} = $self->{tempPic} if ! $self->{curInfo}->{boxpic};
                    $self->{insideInfo} = 0;
                }
            }
            elsif ($tagname eq "a")
            {
                $self->{currentHref} = $attr->{href};

                if ($self->{insideCategories})
                {
                    $self->{insideCategory} = 1; 
                }
                elsif ($self->{insideMechanics})
                {
                    $self->{insideMechanic} = 1;
                }
                elsif ($self->{insideExpandsList})
                {
                    $self->{insideExpands} = 1;
                }
                elsif ($self->{insideExpansions})
                {
                    $self->{insideExpansion} = 1;
                }
            }
            if ($self->{insideDescription})
            {
                if ($tagname eq "br")
                {
                    # neatens up the description a little by starting new line on br tags
                    $self->{curInfo}->{description} .= "\n";
                }
                elsif ($tagname eq "li")
                {
                    # basic formatting of lists
                    $self->{curInfo}->{description} .= " - ";
                }
            }
        }
    }

    sub end
    {
        my ($self, $tagname) = @_;	
        $self->{inside}->{$tagname}--;

	if (($self->{isInsideResult}) && ($tagname eq "td"))
        {
            $self->{isInsideResult} = 0;
        }



	if (($self->{isBoardgame}) && ($tagname eq "a"))
        {
            $self->{itemsList}[$self->{itemIdx}]->{name} = $self->{temporaryName};
            $self->{isBoardgame} = 0;
        }
        if (($tagname eq "td") && ($self->{insideDesigners})) 
        {
            $self->{curInfo}->{designedby} =~ s/(, )$//;
	    $self->{insideDesigners} = 0;
        }
        if (($tagname eq "td") && ($self->{insidePublishers})) 
        {
            $self->{curInfo}->{publishedby} =~ s/(, )$//;
	    $self->{insidePublishers} = 0;
        }
        if (($tagname eq "td") && ($self->{insideArtists})) 
        {
            $self->{curInfo}->{illustratedby} =~ s/(, )$//;
	    $self->{insideArtists} = 0;
        }


        if ($tagname eq "tr")
        {
            $self->{insideCategories} = 0;
            $self->{insideMechanics} = 0;
            $self->{insideExpansions} = 0;
        }
        if ($tagname eq "ul")
        {
            $self->{insideExpandsList} = 0;
        }
        if ($tagname eq "div")
        {
            $self->{insideDescription} = 0;
            # remove spaces from start and end of description
            $self->{curInfo}->{description} =~ s/^\s+//;
            $self->{curInfo}->{description} =~ s/\s+$//;
        }
        if ($self->{insideDescription})
        {
            if (($tagname eq "p") || ($tagname eq "h1") || ($tagname eq "h2") || ($tagname eq "h3") || ($tagname eq "h4")) 
            {
                # neatens up the description a little by placing blank lines between paragraphs
                $self->{curInfo}->{description} .= "\n\n";
            }
            elsif ($tagname eq "li")
            {
                # put new line at end of li elements
                $self->{curInfo}->{description} .= "\n";      
            }
            elsif (($tagname eq "ul") || ($tagname eq "ol"))
            {
                # put blank line after ul, ol elements
                $self->{curInfo}->{description} .= "\n";      
            }
        }
    }

    sub text
    {
        my ($self, $origtext) = @_;

        return if ((length($origtext) < 2) && ($origtext ne " ") && (! $self->{insideDescription}));
        
        $origtext =~ s/&#34;/"/g;
        $origtext =~ s/&#179;/3/g;
        $origtext =~ s/&#[0-9]*;//g;
        $origtext =~ s/\n//g;
        $origtext =~ s/^\s{2,//;



        return if ($self->{parsingEnded});
        
        if ($self->{parsingList})
        {
            if ($self->{isBoardgame})
            {
	        $self->{temporaryName} .= $origtext;
            }
            if ($self->{isInsideRelease})
            {
                if ($origtext =~ /\(([0-9]{4,4})\)$/ )
                {
                    $self->{itemsList}[$self->{itemIdx}]->{released} = $1;
                }
                $self->{isInsideRelease} = 0;
            }
        }
        else
        {

            if ((($origtext =~ /^Expands/) && ($self->{inside}->{strong})) && ($self->{insideDescription}))
            {
		$self->{insideExpandsList} = 1;
            }

            
            if ($self->{insideName})
            {
                $self->{curInfo}->{name} = $origtext;
                $self->{insideName} = 0;
            }
            elsif ($self->{insideYear})
            {
                $self->{curInfo}->{released} = $origtext;
                $self->{curInfo}->{released} =~ s/([^0-9])//g;
                $self->{insideYear} = 0;
            }
            elsif ($self->{insideDesigner})
            {
                $self->{curInfo}->{designedby} .= $origtext.', ';
                $self->{insideDesigner} = 0;
            } 
            elsif ($self->{insidePublisher})
            {
                $self->{curInfo}->{publishedby} .= $origtext.', ';
                $self->{insidePublisher} = 0;
            }
            elsif ($self->{insideArtist})
            {
                $self->{curInfo}->{illustratedby} .= $origtext.', ';
                $self->{insideArtist} = 0;
            }
            elsif ($self->{insidePlayers})
            {
                $self->{curInfo}->{players} = $origtext;
                $self->{curInfo}->{players} =~ s/\s//g;
                $self->{insidePlayers} = 0;
            }
            elsif ($self->{insidePlayingTime})
            {
                $self->{curInfo}->{playingtime} = $origtext;
                $self->{curInfo}->{playingtime} =~ s/\s*$//;
                $self->{curInfo}->{playingtime} =~ s/^\s*//;
                $self->{insidePlayingTime} = 0;
            }
            elsif ($self->{insideAges})
            {
                $self->{curInfo}->{suggestedage} = $origtext;
                $self->{curInfo}->{suggestedage} =~ s/\s*$//;
                $self->{curInfo}->{suggestedage} =~ s/^\s*//;
                $self->{insideAges} = 0;
            }
            elsif ($self->{insideCategory})
            {
                $self->{curInfo}->{category} .= $self->capWord($origtext).',';
                $self->{insideCategory} = 0;
            }
            elsif ($self->{insideMechanic})
            {
                $self->{curInfo}->{mechanics} .= $self->capWord($origtext).',';
                $self->{insideMechanic} = 0;
            }
            elsif ($self->{insideExpands})
            {
                $self->{curInfo}->{expansionfor} = $origtext;
                $self->{insideExpands} = 0;
            }
            elsif ($self->{insideExpansion})
            {
                $self->{tempExpansionName} = $origtext;
                $self->{tempExpansionName} =~ s/\s*$//;
                $self->{tempExpansionName} =~ s/^\s*//;
                $self->{curInfo}->{expandedby} .= $self->capWord($self->{tempExpansionName}).',';
                $self->{insideExpansion} = 0;
            }
            if ($self->{insideDescription})
            {
                $self->{curInfo}->{description} .= $origtext;
            }
        }
    }

    sub new
    {
        my $proto = shift;
        my $class = ref($proto) || $proto;
        my $self  = $class->SUPER::new();
        bless ($self, $class);

        $self->{hasField} = {
            name => 1,
            released => 1,
        };

        $self->{isBoardgame} = 0;
        $self->{curName} = undef;
        $self->{curUrl} = undef;

        return $self;
    }

    sub preProcess
    {
        my ($self, $html) = @_;
        
        $self->{parsingEnded} = 0;
        
        $html =~ s/"&#34;/'"/g;
        $html =~ s/&#34;"/"'/g;
        $html =~ s//'/g;
        $html =~ s//.../g;
        $html =~ s|</a></b><br>|</a><br>|;
             
        return $html;
    }

    sub getSearchUrl
    {
		my ($self, $word) = @_;
		return "http://www.boardgamegeek.com/geeksearch.php?action=search&objecttype=boardgame&q=$word&B1=Go";
    }
    
    sub getItemUrl
    {
		my ($self, $url) = @_;
		
		return $url if $url =~ /^http:/;
		return "http://www.boardgamegeek.com".$url;
    }

    sub getName
    {
        return "Board Game Geek";
    }
    
    sub getAuthor
    {
        return 'Zombiepig';
    }
    
    sub getLang
    {
        return 'EN';
    }
    
    
}

1;
