package GCPlugins::GCfilms::GCAllocine;

###################################################
#
#  Copyright 2005-2009 Tian
#
#  This file is part of GCstar.
#
#  GCstar is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  GCstar is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with GCstar; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###################################################

use strict;
use utf8;

use GCPlugins::GCfilms::GCfilmsCommon;

{
    package GCPlugins::GCfilms::GCPluginAllocine;

    use base qw(GCPlugins::GCfilms::GCfilmsPluginsBase);
 
    sub start
    {
        my ($self, $tagname, $attr, $attrseq, $origtext) = @_;        
        $self->{inside}->{$tagname}++;

        if ($self->{parsingList})
        {
            if ($tagname eq "a")
            {

                if (($attr->{href} =~ /^\/film\/fichefilm_gen_cfilm=/) && ($self->{inside}->{h4}))
                {
                    my $url = $attr->{href};
                    $self->{isMovie} = 1;
                    $self->{isInfo} = 0;
                    $self->{itemIdx}++;
                    $self->{itemsList}[$self->{itemIdx}]->{url} = $url;
                }
            }
        }
        else
        {
            if ($tagname eq "h3")
            {
                $self->{insideActors} = 0;
                $self->{insideDirector} = 0;
                $self->{insideCast}=0;
                $self->{insideCastLine}=0;
                $self->{insideRole}=0;
                $self->{insideActor}=0;
                $self->{insideDate}=0;
                if ($attr->{style} =~ /color:\s*#D20000/)
                {
                    $self->{insideAge} = 1;
                }
                else
                {
                    $self->{insideAge} = 0;
                }
            }
            elsif ($tagname eq "img")
            {
                my $src = $attr->{src};
                if($self->{insidePressRating} == 1 && $attr->{class}=~/etoile_(\d)/){
                    $self->{curInfo}->{ratingpress}=2+2*$1;
                    $self->{insidePressRating} = 0;
                }
                elsif (!$self->{curInfo}->{image})
                {
                    if ($src =~ /\/nmedia\//)
                    {
                        if ($self->{bigPics})
                        {
                            my $pageUrl = $self->{loadedUrl};
                            $pageUrl =~ s/fichefilm_/galerievignette_/;
                            my $page = $self->loadPage($pageUrl, 0, 1);
                            $page =~ m/<img id='imgNormal' class='photo' src='(.*?)'/ms;
                            $src = $1 if $1;
                        }
                       $self->{curInfo}->{image} = $src;
                    }
                    elsif ($src =~ /AffichetteAllocine\.gif/i)
                    {
                        $self->{curInfo}->{image} = "empty";
                    }
               }
           }
           elsif (($tagname eq "table") && ($attr->{cast} eq "1"))
           {
               $self->{insideCast}=1;
           }
           elsif (($tagname eq "tr") && ($self->{insideCast}))
           {
               $self->{insideCastLine}=1;
           }
           elsif (($tagname eq "h5") && ($self->{insideCastLine}))
           {
               $self->{insideRole}=1;
           }
           elsif (($tagname eq "a") && ($self->{insideCastLine}))
           {
               $self->{insideRole}=0;
               $self->{insideActor}=1;
           }
           elsif ($tagname eq "br")
           {
               if ($self->{insideSynopsis})
               {
                   $self->{curInfo}->{synopsis} .= "\n";
               }
           }
        }
    }

    sub end
    {
        my ($self, $tagname) = @_;
        $self->{inside}->{$tagname}--;

        if ($tagname eq "h4")
        {
            $self->{insideSynopsis} = 0;
        }
       elsif (($tagname eq "td") && ($self->{isInfo}))
       {
           #work around bad date for last search result
           $self->{isInfo}=0;
       }
       elsif (($tagname eq "td") && ($self->{insideCastLine}))
       {
           $self->{insideRole}=0;
           $self->{insideActor}=0;
       }
       elsif (($tagname eq "tr") && ($self->{insideCastLine}))
       {
           $self->{insideCastLine}=0;
       }
       elsif (($tagname eq "table") && ($self->{insideCast}))
       {
           $self->{insideCast}=0;
       }
    }

    sub text
    {
        my ($self, $origtext) = @_;

        if ($self->{parsingList})
        {
            if ($self->{isMovie})
            {
                $self->{itemsList}[$self->{itemIdx}]->{title} = $origtext;
                $self->{isMovie} = 0;
                $self->{isInfo} = 1;
                return;
            }
            if ($self->{isInfo})
            {
                if ($self->{inside}->{h4})
                {
                    if ( $origtext =~ /([0-9]{4})/ )
                    {
                        $self->{itemsList}[$self->{itemIdx}]->{date} = $1;
                    }
                }
                elsif ($self->{inside}->{h5})
                {
                    if ( $origtext =~ /^de (.*)/ )
                    {
                        $self->{itemsList}[$self->{itemIdx}]->{director} = $1;
                    }
                    elsif (( $origtext =~ m/^avec (.*)/ ) && (!$self->{itemsList}[$self->{itemIdx}]->{actors}))
                    {
                        $self->{itemsList}[$self->{itemIdx}]->{actors} = $1;
                    }
                }
            }
        }
        else
        {
            if (($self->{inside}->{h1}) && !($self->{inside}->{script}))
            {
                $origtext =~ s/[\r\n]//g;
                $self->{curInfo}->{title} = $origtext if length($origtext) > 2;
            }
            if ($self->{inside}->{h2})
            {
                $self->{insideSynopsis} = 1 if $origtext =~ /Synopsis/;
            }
            if ($self->{inside}->{h4}){
                if ($self->{insideActors})
                {
                    $self->{curInfo}->{synopsis} .= $origtext;
                }
                elsif ($self->{insideSynopsis})
                {
                    $self->{curInfo}->{synopsis} .= $origtext;
                }
                elsif ( $origtext =~ /^Saga /)
                {
                    $origtext =~ s/Saga //;
                    $self->{curInfo}->{serie}=$origtext;
                }
            }
            elsif ($self->{inside}->{h3})
            {
                if ($self->{insideActors})
                {
                    #Actors from fichefilm page
                    $self->{curInfo}->{actors} .= $origtext;
                    $self->{curInfo}->{actors} =~ s/^\s*//;
                }
                elsif ($self->{insideRole})
                {
                    #Roles from casting page
                    push @{$self->{curInfo}->{actors}}, [$origtext,$origtext] 
                    if ($self->{actorsCounter} <= $GCPlugins::GCfilms::GCfilmsCommon::MAX_ACTORS);
                }
                elsif ($self->{insideActor})
                {
                    #Actors from casting page
                    $self->{curInfo}->{actors}->[$self->{actorsCounter}]->[0]=$origtext
                    if ($self->{actorsCounter} <= $GCPlugins::GCfilms::GCfilmsCommon::MAX_ACTORS);
                    $self->{actorsCounter}++;
                }
                elsif ($self->{insideDirector})
                {
                    $self->{curInfo}->{director} .= $origtext;
                }
                elsif ($self->{insideAge})
                {
                    $origtext =~ s/.*?([0-9]+).*/$1/;
                    $self->{curInfo}->{age} = $origtext;
                }
                elsif ($self->{insideDate})
                {
                    $origtext =~ s/^[\s\n]*//;
                    $origtext =~ s/\s*$//;
                    $self->{curInfo}->{date} = $origtext;
                    $self->{insideDate}=0;
                }
                elsif (( $origtext =~ /^Date de sortie/ )
                    && ( !$self->{curInfo}->{date} ) )
                {
                    $self->{insideDate}=1;
                }
                elsif ( $origtext =~ /^Genre/ )
                {
                    $origtext =~ s/^Genre : //;
                    $origtext =~ s/, /,/g;
                    $self->{curInfo}->{genre} = $self->capWord($origtext);
                }
                elsif (( $origtext =~ /^Film / ) && (!$self->{curInfo}->{country}))
                {
                    $origtext =~ s/^Film //;
                    $self->{curInfo}->{country} = ucfirst $origtext;
                }
                elsif ( $origtext =~ /^Dur.e/ )
                {
                    $origtext =~ s/^Dur.e : (.*)?/$1/;
                    $self->{curInfo}->{time} = $origtext;
                }
                elsif ( $origtext =~ /^Titre original/ )
                {
                    $origtext =~ s/^Titre original : .*?/$1/;
                    $self->{curInfo}->{original} = $origtext;
                }
                elsif (($origtext =~ /^Avec/ ) && (! $self->{curInfo}->{actors}))
                {
                    #Detecting actors from fichefilm page
                    $self->{insideActors} = 1;
                }
                elsif ( $origtext =~ /^R.alis. par/ )
                {
                    $self->{insideDirector} = 1;
                }
            }
            elsif ( $origtext =~ /^Presse$/ )
            {
               $self->{insidePressRating} = 1;
            }
        }
    }

    sub new
    {
        my $proto = shift;
        my $class = ref($proto) || $proto;
        my $self  = $class->SUPER::new();

        $self->{hasField} = {
            title => 1,
            date => 1,
            director => 1,
            actors => 1,
        };

        $self->{isInfo} = 0;
        $self->{isMovie} = 0;
        $self->{curName} = undef;
        $self->{curUrl} = undef;
        $self->{actorsCounter}=0;

        bless ($self, $class);
        return $self;
    }

    sub preProcess
    {
        my ($self, $html) = @_;
        $html =~ s/&nbsp;/ /gi;
        $html =~ s/<b>|<strong>|<\/b>|<\/strong>|<i>|<\/i>//gi;
        
        #Remove links for genre
        $html =~ s|<a href="/film/alaffiche_genre_gen_genre.*?>(.*?)</a>|$1|gi;
        #"

        if (!$self->{parsingList})
        {
            #Remplacing actor list with cast page list
            my $pageUrl = $self->{loadedUrl};
            $pageUrl =~ s/fichefilm_/casting_/;
            my $page = $self->loadPage($pageUrl, 0, 1);
            if($page =~ m|<h2[^>]*>Acteurs</h2><hr /></td></tr>(.*?)</table>|ms)
            {
                my $src = $1;
                $html =~ s|Avec.*Plus\.\.\.</a>|<table cast=1>$src</table>|i;
            }
        }
        
        return $html;
    }
    
    sub getSearchUrl
    {
        my ($self, $word) = @_;
	
        return "http://www.allocine.fr/recherche/?motcle=$word&f=3&rub=1";
    }
    
    sub getItemUrl
    {
        my ($self, $url) = @_;
		
        return "http://www.allocine.fr".$url;
    }

    sub getName
    {
        return "Allocine.fr";
    }
    
    sub getAuthor
    {
        return 'Tian';
    }
    
    sub getLang
    {
        return 'FR';
    }
    
    sub getCharset
    {
        return "ISO-8859-1";
    }
}

1;
